#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 1/21/20

# Comments directed to http://gimpchat.com

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

# Change log:
# Version 2 - Added a few options: Background color, Noise and Round shape


from gimpfu import *
import os, string, sys,time
import gettext
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_echo_trail_text_v2(customtext, fontname, t_color, b_color, noise, value, rounded):

    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
    
    size = 190
    img = pdb.gimp_image_new(1, 1, RGB)
    pdb.gimp_context_set_foreground(t_color)
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 0, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 10)
    #Resize text
    img.resize(textLayer.width, textLayer.height, 0, 0)
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = (img.width - textLayer.width) / 2
    y = (img.height - textLayer.height) / 2
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    pdb.gimp_display_new(img)
    
    #Create background
    marginX = 10
    marginY = 225
    pdb.gimp_image_resize(img, img.width+2*marginX, img.height+2*marginY, marginX, marginY)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, backLayer, None, 1)
    pdb.gimp_context_set_background(b_color)
    pdb.gimp_drawable_fill(backLayer, FILL_BACKGROUND)
    
    #Use noise option
    if noise==True:
        pdb.plug_in_hsv_noise(img, backLayer, 1, 3, 100, value)
        pdb.plug_in_gauss(img, backLayer, 2, 2, 1)
        
    #Create alightly outer glow on text    
    pdb.script_fu_drop_shadow(img, textLayer, 0, 0, 25, "#000000", 80, FALSE)
    shadLayer = pdb.gimp_image_get_layer_by_name(img, "Drop Shadow")
    shadLayer = pdb.gimp_image_merge_down(img, textLayer, CLIP_TO_IMAGE)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Drop Shadow")
    pdb.gimp_item_set_name(shadLayer, "Text")
        
    #Offsets variables
    rendAmount_1 = 5
    counter1 = rendAmount_1
    rendAmount_2 = 5
    counter2 = rendAmount_2
    rendDir_x1 = 0
    rendDir_y1 = -45
    rendDir_x = 0
    rendDir_y = 0
    rendDir_x2 = 0
    rendDir_y2 = 45
      
    #North
    rendDir_x = 0
    rendDir_y = -1
    
    #south
    rendDir_x3 = 0
    rendDir_y3 = 1
    
    #Calculate
    rendDir_x1 = rendDir_x1 + rendDir_x
    rendDir_y1 = rendDir_y1 + rendDir_y 
    rendDir_x3 = rendDir_x3 + rendDir_x2
    rendDir_y3 = rendDir_y3 + rendDir_x2 
    
    #Get layer dimensions
    width = pdb.gimp_drawable_width(textLayer)
    height = pdb.gimp_drawable_height(textLayer)     
        
    #Copy the text for the top motion
    rendLayer = pdb.gimp_layer_new_from_drawable(textLayer, img)
    pdb.gimp_image_insert_layer(img, rendLayer, None, 1)
    pdb.gimp_layer_set_mode(rendLayer, LAYER_MODE_NORMAL)
    pdb.gimp_item_set_name(rendLayer, "Top Ripple") 
    rendLayer = pdb.gimp_image_get_layer_by_name(img, "Top Ripple")
       
    #Create the top motion
    pdb.gimp_context_set_foreground("#000000")
    width = pdb.gimp_image_width(img)
    height = pdb.gimp_image_height(img)
    while (counter1 > 0):    
        counter1 = counter1 - 1
        rendLayer2 = pdb.gimp_layer_new_from_drawable(rendLayer, img)
        pdb.gimp_image_insert_layer(img, rendLayer2, None, 1)
        pdb.gimp_layer_set_mode(rendLayer2, LAYER_MODE_NORMAL)
        pdb.gimp_drawable_offset(rendLayer2, FALSE, 2, rendDir_x1, rendDir_y1)
        mask = pdb.gimp_layer_create_mask(rendLayer2, ADD_MASK_WHITE)
        pdb.gimp_layer_add_mask(rendLayer2, mask)
        pdb.gimp_edit_blend(mask, 2, LAYER_MODE_NORMAL, 0, 100, 0, 0, TRUE, TRUE, 3, 0.20, TRUE, img.width / 2, 0-200, img.width / 2, img.height - 200)
        pdb.gimp_layer_set_apply_mask(rendLayer2, TRUE)
        rendLayer = pdb.gimp_image_merge_down(img, rendLayer2, CLIP_TO_IMAGE)
           
    #Copy the text for the bottom motion
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")    
    rendLayer3 = pdb.gimp_layer_new_from_drawable(textLayer, img)
    pdb.gimp_image_insert_layer(img, rendLayer3, None, 2)
    pdb.gimp_layer_set_mode(rendLayer3, LAYER_MODE_NORMAL)
    pdb.gimp_item_set_name(rendLayer3, "Bottom Ripple") 
    rendLayer = pdb.gimp_image_get_layer_by_name(img, "Bottom Ripple")
    
    #Create bottom motion
    while (counter2 > 0):    
        counter2 = counter2 - 1
        rendLayer3 = pdb.gimp_layer_new_from_drawable(rendLayer, img)
        pdb.gimp_image_insert_layer(img, rendLayer3, None, 2)
        pdb.gimp_layer_set_mode(rendLayer3, LAYER_MODE_NORMAL)
        pdb.gimp_drawable_offset(rendLayer3, FALSE, 2, rendDir_x2, rendDir_y2)
        mask = pdb.gimp_layer_create_mask(rendLayer3, ADD_MASK_WHITE)
        pdb.gimp_layer_add_mask(rendLayer3, mask)
        pdb.gimp_edit_blend(mask, 2, LAYER_MODE_NORMAL, 0, 100, 0, 0, FALSE, TRUE, 3, 0.20, TRUE, img.width / 2, img.height / 2, img.width / 2, img.height - 100)
        pdb.gimp_layer_set_apply_mask(rendLayer3, TRUE)
        rendLayer = pdb.gimp_image_merge_down(img, rendLayer3, CLIP_TO_IMAGE)  
        
    #Merge all text layers
    pdb.gimp_item_set_visible(backLayer, FALSE)
    textLayer = pdb.gimp_image_merge_visible_layers(img, EXPAND_AS_NECESSARY)
    
    #Make a cilider shape with text 
    factor = 150
    if rounded==True:
        textLayer.scale(textLayer.width, textLayer.height - factor)
        textLayer.set_offsets((img.width - textLayer.width) / 2, (img.height - textLayer.height) / 2)
        pdb.gimp_layer_resize_to_image_size(textLayer)
        pdb.plug_in_gmic_qt(img, textLayer, 1, 0, "-gui_rep_polkal 1,0.9,0,50,50,0,1,0,0,0,1,0.5,2")
    pdb.gimp_item_set_visible(backLayer, TRUE)
      
    #Resets the previous user defaults
    pdb.gimp_context_pop() 
    

def pm_echo_trail_text_atl_v2(img, layer, t_color, b_color, noise, value, rounded):    
  
    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
    
    #Make the background active
    img.layers[-1]
    backLayer = img.layers[-1]
    pdb.gimp_image_set_active_layer(img, backLayer)
    pdb.gimp_item_set_name(backLayer, "Background")
    pdb.gimp_context_set_foreground(b_color)
    pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND)
    
    #Use noise option
    if noise==True:
        pdb.plug_in_hsv_noise(img, backLayer, 1, 3, 100, value)
        pdb.plug_in_gauss(img, backLayer, 2, 2, 1)
 
    #Make the text active
    img.layers[0]
    textLayer = img.layers[0]
    pdb.gimp_image_set_active_layer(img, textLayer)
    
    #Option for text color
    pdb.gimp_image_set_active_layer(img, textLayer)
    pdb.gimp_item_set_name(textLayer, "Text")
    textlayer = pdb.gimp_item_is_text_layer(textLayer)
    pdb.gimp_context_set_foreground(t_color)
    pdb.gimp_context_set_background(t_color)
    pdb.gimp_context_set_gradient("FG to BG (RGB)")
    pdb.plug_in_gradmap(img, textLayer)
           
    #Center the text to the canvas
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = 0
    y = (img.height - textLayer.height) / 2
    textLayer.scale(backLayer.width, textLayer.height)
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    #Create alightly outer glow on text    
    pdb.script_fu_drop_shadow(img, textLayer, 0, 0, 25, "#000000", 80, FALSE)
    shadLayer = pdb.gimp_image_get_layer_by_name(img, "Drop Shadow")
    shadLayer = pdb.gimp_image_merge_down(img, textLayer, CLIP_TO_IMAGE)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Drop Shadow")
    pdb.gimp_item_set_name(shadLayer, "Text")
        
    #Offsets variables
    rendAmount_1 = 5
    counter1 = rendAmount_1
    rendAmount_2 = 5
    counter2 = rendAmount_2
    rendDir_x1 = 0
    rendDir_y1 = -45
    rendDir_x = 0
    rendDir_y = 0
    rendDir_x2 = 0
    rendDir_y2 = 45
      
    #North
    rendDir_x = 0
    rendDir_y = -1
    
    #south
    rendDir_x3 = 0
    rendDir_y3 = 1
    
    #Calculate
    rendDir_x1 = rendDir_x1 + rendDir_x
    rendDir_y1 = rendDir_y1 + rendDir_y 
    rendDir_x3 = rendDir_x3 + rendDir_x2
    rendDir_y3 = rendDir_y3 + rendDir_x2 
    
    #Get layer dimensions
    width = pdb.gimp_drawable_width(textLayer)
    height = pdb.gimp_drawable_height(textLayer)     
        
    #Copy the text for the top motion
    rendLayer = pdb.gimp_layer_new_from_drawable(textLayer, img)
    pdb.gimp_image_insert_layer(img, rendLayer, None, 1)
    pdb.gimp_layer_set_mode(rendLayer, LAYER_MODE_NORMAL)
    pdb.gimp_item_set_name(rendLayer, "Top Ripple") 
    rendLayer = pdb.gimp_image_get_layer_by_name(img, "Top Ripple")
       
    #Create the top motion
    pdb.gimp_context_set_foreground("#000000")
    width = pdb.gimp_image_width(img)
    height = pdb.gimp_image_height(img)
    while (counter1 > 0):    
        counter1 = counter1 - 1
        rendLayer2 = pdb.gimp_layer_new_from_drawable(rendLayer, img)
        pdb.gimp_image_insert_layer(img, rendLayer2, None, 1)
        pdb.gimp_layer_set_mode(rendLayer2, LAYER_MODE_NORMAL)
        pdb.gimp_drawable_offset(rendLayer2, FALSE, 2, rendDir_x1, rendDir_y1)
        mask = pdb.gimp_layer_create_mask(rendLayer2, ADD_MASK_WHITE)
        pdb.gimp_layer_add_mask(rendLayer2, mask)
        pdb.gimp_edit_blend(mask, 2, LAYER_MODE_NORMAL, 0, 100, 0, 0, TRUE, TRUE, 3, 0.20, TRUE, img.width / 2, 0-200, img.width / 2, img.height - 200)
        pdb.gimp_layer_set_apply_mask(rendLayer2, TRUE)
        rendLayer = pdb.gimp_image_merge_down(img, rendLayer2, CLIP_TO_IMAGE)
           
    #Copy the text for the bottom motion
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")    
    rendLayer3 = pdb.gimp_layer_new_from_drawable(textLayer, img)
    pdb.gimp_image_insert_layer(img, rendLayer3, None, 2)
    pdb.gimp_layer_set_mode(rendLayer3, LAYER_MODE_NORMAL)
    pdb.gimp_item_set_name(rendLayer3, "Bottom Ripple") 
    rendLayer = pdb.gimp_image_get_layer_by_name(img, "Bottom Ripple")
    
    #Create bottom motion
    while (counter2 > 0):    
        counter2 = counter2 - 1
        rendLayer3 = pdb.gimp_layer_new_from_drawable(rendLayer, img)
        pdb.gimp_image_insert_layer(img, rendLayer3, None, 2)
        pdb.gimp_layer_set_mode(rendLayer3, LAYER_MODE_NORMAL)
        pdb.gimp_drawable_offset(rendLayer3, FALSE, 2, rendDir_x2, rendDir_y2)
        mask = pdb.gimp_layer_create_mask(rendLayer3, ADD_MASK_WHITE)
        pdb.gimp_layer_add_mask(rendLayer3, mask)
        pdb.gimp_edit_blend(mask, 2, LAYER_MODE_NORMAL, 0, 100, 0, 0, FALSE, TRUE, 3, 0.20, TRUE, img.width / 2, img.height / 2, img.width / 2, img.height - 100)
        pdb.gimp_layer_set_apply_mask(rendLayer3, TRUE)
        rendLayer = pdb.gimp_image_merge_down(img, rendLayer3, CLIP_TO_IMAGE)        
    
    #Merge all text layers
    pdb.gimp_item_set_visible(backLayer, FALSE)
    textLayer = pdb.gimp_image_merge_visible_layers(img, EXPAND_AS_NECESSARY)
    
    #Make a cilider shape with text 
    factor = 150
    if rounded==True:
        textLayer.scale(textLayer.width, textLayer.height - factor)
        textLayer.set_offsets((img.width - textLayer.width) / 2, (img.height - textLayer.height) / 2)
        pdb.gimp_layer_resize_to_image_size(textLayer)
        pdb.plug_in_gmic_qt(img, textLayer, 1, 0, "-gui_rep_polkal 1,0.9,0,50,50,0,1,0,0,0,1,0.5,2")
    pdb.gimp_item_set_visible(backLayer, TRUE)
        
    #Process
    pdb.gimp_displays_flush()
    
    #Resets the previous user defaults
    pdb.gimp_context_pop() 
    
register(
    "pm_echo_trail_text_atl_v2",
    "Render a repeated text top and bottom",
    "Render a repeated text top and bottom",
    "Pocholo M",
    "Pocholo M",
    "2022",
    "Echo Trail Text Effect V2",
    "",
    [
    (PF_IMAGE,      "img", "Input image", None),
    (PF_DRAWABLE,   "drawable", "Input layer", None),
    (PF_COLOR, "t_color", "Text Color", (255, 255, 255)),
    (PF_COLOR, "B_color", "Background Color", (000, 000, 000)),
    (PF_TOGGLE, "noise", "Add Noise?:", 0),
    (PF_SPINNER, "value", "Noise Increase", 25, (1, 50, 1)),
    (PF_TOGGLE, "rounded", "Text Cilinder?:", 0),
    ],
    [],
    pm_echo_trail_text_atl_v2, menu="<Image>/Filters/Alpha to Logo",
    domain=("gimp20", gimp.locale_directory))
       

register(
    "pm_echo_trail_text_v2",
    "Render a repeated text top and bottom",
    "Render a repeated text top and bottom",
    "Pocholo M",
    "Pocholo M",
    "2022",
    "Echo Trail Text Effect V2",
    "",
    [
    (PF_STRING, "customtext", "Text string", "ECHOTEXT"),
    (PF_FONT, "fontname", "Font name", "Arial Bold"),
    (PF_COLOR, "t_color", "Text Color", (255, 255, 255)),
    (PF_COLOR, "B_color", "Background Color", (0, 0, 0)),
    (PF_TOGGLE, "noise", "Add Noise?:", 0),
    (PF_SPINNER, "value", "Noise Increase", 25, (1, 50, 1)),
    (PF_TOGGLE, "rounded", "Text Cilinder?:", 0),
    
    ],
    [],
    pm_echo_trail_text_v2, menu="<Image>/Python-Fu/Text Effects",
    domain=("gimp20", gimp.locale_directory))
main()			       